@@ -27,9 +27,9 @@ def main():
             label_name_to_value[label_name] = label_value
     lbl, _ = utils.shapes_to_label(img.shape, label_file.shapes, label_name_to_value)
 
-    label_names = [None] * (max(label_name_to_value.values()) + 1)
+    label_names: list[str] = [""] * (max(label_name_to_value.values()) + 1)
     for name, value in label_name_to_value.items():
-        label_names[value] = name  # type: ignore[call-overload]
+        label_names[value] = name
     lbl_viz = imgviz.label2rgb(
         lbl,
         imgviz.asgray(img),
